@@ -2,6 +2,7 @@
 * TLS ASIO Stream
 * (C) 2018-2021 Jack Lloyd
 *     2018-2021 Hannes Rantzsch, Tim Oesterreich, Rene Meusel
+*     2023      Fabian Albert, René Meusel - Rohde & Schwarz Cybersecurity
 *
 * Botan is released under the Simplified BSD License (see license.txt)
 */
@@ -9,11 +10,10 @@
 #ifndef BOTAN_ASIO_STREAM_H_
 #define BOTAN_ASIO_STREAM_H_
 
-#include <botan/types.h>
-
-// first version to be compatible with Networking TS (N4656) and boost::beast
-#include <boost/version.hpp>
-#if BOOST_VERSION >= 106600
+#include <botan/asio_compat.h>
+#if !defined(BOTAN_FOUND_COMPATIBLE_BOOST_ASIO_VERSION)
+   #error Available boost headers are too old for the boost asio stream.
+#else
 
    #include <botan/asio_async_ops.h>
    #include <botan/asio_context.h>
@@ -35,8 +35,119 @@
    #include <memory>
    #include <type_traits>
 
-namespace Botan {
-namespace TLS {
+namespace Botan::TLS {
+
+template <class SL, class C>
+class Stream;
+
+/**
+ * @brief Specialization of TLS::Callbacks for the ASIO Stream
+ *
+ * Applications may decide to derive from this for fine-grained customizations
+ * of the TLS::Stream's behaviour. Examples may be OCSP integration, custom
+ * certificate validation or user-defined key exchange mechanisms.
+ *
+ * By default, this class provides all necessary customizations for the ASIO
+ * integration. The methods required for that are `final` and cannot be
+ * overridden.
+ *
+ * Each instance of TLS::Stream must have their own instance of this class. A
+ * future major version of Botan will therefor consume instances of this class
+ * as a std::unique_ptr. The current usage of std::shared_ptr is erratic.
+ */
+class StreamCallbacks : public Callbacks {
+   public:
+      StreamCallbacks() {}
+
+      void tls_emit_data(std::span<const uint8_t> data) final {
+         m_send_buffer.commit(boost::asio::buffer_copy(m_send_buffer.prepare(data.size()),
+                                                       boost::asio::buffer(data.data(), data.size())));
+      }
+
+      void tls_record_received(uint64_t, std::span<const uint8_t> data) final {
+         m_receive_buffer.commit(boost::asio::buffer_copy(m_receive_buffer.prepare(data.size()),
+                                                          boost::asio::const_buffer(data.data(), data.size())));
+      }
+
+      bool tls_peer_closed_connection() final {
+         // Instruct the TLS implementation to reply with our close_notify to
+         // obtain the same behaviour for TLS 1.2 and TLS 1.3. Currently, this
+         // prevents a downstream application from closing their write-end while
+         // allowing the peer to continue writing.
+         //
+         // When lifting this limitation, please take good note of the "Future
+         // work" remarks in https://github.com/randombit/botan/pull/3801.
+         return true;
+      }
+
+      /**
+       * @param alert  a TLS alert sent by the peer
+       */
+      void tls_alert(TLS::Alert alert) final {
+         if(alert.is_fatal() || alert.type() == TLS::AlertType::CloseNotify) {
+            // TLS alerts received from the peer are not passed to the
+            // downstream application immediately. Instead, we retain them here
+            // and the stream invokes `handle_tls_protocol_errors()` in due time
+            // to handle them.
+            m_alert_from_peer = alert;
+         }
+      }
+
+      void tls_verify_cert_chain(const std::vector<X509_Certificate>& cert_chain,
+                                 const std::vector<std::optional<OCSP::Response>>& ocsp_responses,
+                                 const std::vector<Certificate_Store*>& trusted_roots,
+                                 Usage_Type usage,
+                                 std::string_view hostname,
+                                 const TLS::Policy& policy) override {
+         auto ctx = m_context.lock();
+
+         if(ctx && ctx->has_verify_callback()) {
+            ctx->get_verify_callback()(cert_chain, ocsp_responses, trusted_roots, usage, hostname, policy);
+         } else {
+            Callbacks::tls_verify_cert_chain(cert_chain, ocsp_responses, trusted_roots, usage, hostname, policy);
+         }
+      }
+
+   private:
+      // The members below are meant for the tightly-coupled Stream class only
+      template <class SL, class C>
+      friend class Stream;
+
+      void set_context(std::weak_ptr<Botan::TLS::Context> context) { m_context = std::move(context); }
+
+      void consume_send_buffer() { m_send_buffer.consume(m_send_buffer.size()); }
+
+      boost::beast::flat_buffer& send_buffer() { return m_send_buffer; }
+
+      const boost::beast::flat_buffer& send_buffer() const { return m_send_buffer; }
+
+      boost::beast::flat_buffer& receive_buffer() { return m_receive_buffer; }
+
+      const boost::beast::flat_buffer& receive_buffer() const { return m_receive_buffer; }
+
+      bool shutdown_received() const {
+         return m_alert_from_peer && m_alert_from_peer->type() == AlertType::CloseNotify;
+      }
+
+      std::optional<Alert> alert_from_peer() const { return m_alert_from_peer; }
+
+   private:
+      std::optional<Alert> m_alert_from_peer;
+      boost::beast::flat_buffer m_receive_buffer;
+      boost::beast::flat_buffer m_send_buffer;
+
+      std::weak_ptr<TLS::Context> m_context;
+};
+
+namespace detail {
+
+template <typename T>
+concept basic_completion_token = boost::asio::completion_token_for<T, void(boost::system::error_code)>;
+
+template <typename T>
+concept byte_size_completion_token = boost::asio::completion_token_for<T, void(boost::system::error_code, size_t)>;
+
+}  // namespace detail
 
 /**
  * @brief boost::asio compatible SSL/TLS stream
@@ -46,25 +157,45 @@ namespace TLS {
  */
 template <class StreamLayer, class ChannelT = Channel>
 class Stream {
+   private:
+      using default_completion_token =
+         boost::asio::default_completion_token_t<boost::beast::executor_type<StreamLayer>>;
+
    public:
       //! \name construction
       //! @{
 
+      /**
+       * @brief Construct a new Stream with a customizable instance of Callbacks
+       *
+       * @param context The context parameter is used to set up the underlying native handle.
+       * @param callbacks The callbacks parameter may contain an instance of a derived TLS::Callbacks
+       *                  class to allow for fine-grained customization of the TLS stream. Note that
+       *                  applications need to ensure a 1-to-1 relationship between instances of
+       *                  Callbacks and Streams. A future major version of Botan will use a unique_ptr
+       *                  here.
+       *
+       * @param args Arguments to be forwarded to the construction of the next layer.
+       */
+      template <typename... Args>
+      explicit Stream(std::shared_ptr<Context> context, std::shared_ptr<StreamCallbacks> callbacks, Args&&... args) :
+            m_context(std::move(context)),
+            m_nextLayer(std::forward<Args>(args)...),
+            m_core(std::move(callbacks)),
+            m_input_buffer_space(MAX_CIPHERTEXT_SIZE, '\0'),
+            m_input_buffer(m_input_buffer_space.data(), m_input_buffer_space.size()) {
+         m_core->set_context(m_context);
+      }
+
       /**
        * @brief Construct a new Stream
        *
-       * @param context The context parameter is used to set up the underlying native handle. Using code is
-       *                responsible for lifetime management of the context and must ensure that it is available for the
-       *                lifetime of the stream.
+       * @param context The context parameter is used to set up the underlying native handle.
        * @param args Arguments to be forwarded to the construction of the next layer.
        */
       template <typename... Args>
       explicit Stream(std::shared_ptr<Context> context, Args&&... args) :
-            m_context(context),
-            m_nextLayer(std::forward<Args>(args)...),
-            m_core(std::make_shared<StreamCore>(context)),
-            m_input_buffer_space(MAX_CIPHERTEXT_SIZE, '\0'),
-            m_input_buffer(m_input_buffer_space.data(), m_input_buffer_space.size()) {}
+            Stream(std::move(context), std::make_shared<StreamCallbacks>(), std::forward<Args>(args)...) {}
 
       /**
        * @brief Construct a new Stream
@@ -72,17 +203,16 @@ class Stream {
        * Convenience overload for boost::asio::ssl::stream compatibility.
        *
        * @param arg This argument is forwarded to the construction of the next layer.
-       * @param context The context parameter is used to set up the underlying native handle. Using code is
-       *                responsible for lifetime management of the context and must ensure that is available for the
-       *                lifetime of the stream.
+       * @param context The context parameter is used to set up the underlying native handle.
+       * @param callbacks The (optional) callbacks object that the stream should use. Note that
+       *                  applications need to ensure a 1-to-1 relationship between instances of Callbacks
+       *                  and Streams. A future major version of Botan will use a unique_ptr here.
        */
       template <typename Arg>
-      explicit Stream(Arg&& arg, std::shared_ptr<Context> context) :
-            m_context(context),
-            m_nextLayer(std::forward<Arg>(arg)),
-            m_core(std::make_shared<StreamCore>(context)),
-            m_input_buffer_space(MAX_CIPHERTEXT_SIZE, '\0'),
-            m_input_buffer(m_input_buffer_space.data(), m_input_buffer_space.size()) {}
+      explicit Stream(Arg&& arg,
+                      std::shared_ptr<Context> context,
+                      std::shared_ptr<StreamCallbacks> callbacks = std::make_shared<StreamCallbacks>()) :
+            Stream(std::move(context), std::move(callbacks), std::forward<Arg>(arg)) {}
 
       virtual ~Stream() = default;
 
@@ -102,24 +232,11 @@ class Stream {
 
       next_layer_type& next_layer() { return m_nextLayer; }
 
-   #if BOOST_VERSION >= 107000
-      /*
-       * From Boost 1.70 onwards Beast types no longer provide public access to the member function `lowest_layer()`.
-       * Instead, the new free-standing functions in Beast need to be used.
-       * See also: https://github.com/boostorg/beast/commit/6a658b5c3a36f8d58334f8b6582c01c3e87768ae
-       */
       using lowest_layer_type = typename boost::beast::lowest_layer_type<StreamLayer>;
 
       lowest_layer_type& lowest_layer() { return boost::beast::get_lowest_layer(m_nextLayer); }
 
       const lowest_layer_type& lowest_layer() const { return boost::beast::get_lowest_layer(m_nextLayer); }
-   #else
-      using lowest_layer_type = typename next_layer_type::lowest_layer_type;
-
-      lowest_layer_type& lowest_layer() { return m_nextLayer.lowest_layer(); }
-
-      const lowest_layer_type& lowest_layer() const { return m_nextLayer.lowest_layer(); }
-   #endif
 
       using executor_type = typename next_layer_type::executor_type;
 
@@ -128,9 +245,7 @@ class Stream {
       using native_handle_type = typename std::add_pointer<ChannelT>::type;
 
       native_handle_type native_handle() {
-         if(m_native_handle == nullptr) {
-            throw Invalid_State("Invalid handshake state");
-         }
+         BOTAN_STATE_CHECK(m_native_handle != nullptr);
          return m_native_handle.get();
       }
 
@@ -225,21 +340,37 @@ class Stream {
       void handshake(Connection_Side side, boost::system::error_code& ec) {
          setup_native_handle(side, ec);
 
-         if(side == Connection_Side::Client) {
-            // send client hello, which was written to the send buffer on client instantiation
-            send_pending_encrypted_data(ec);
-         }
+         // We write to the socket if we have data to send and read from it
+         // otherwise, until either some error occured or we have successfully
+         // performed the handshake.
+         while(!ec) {
+            // Send pending data to the peer and abort the handshake if that
+            // fails with a network error. We do that first, to allow sending
+            // any final message before reporting the handshake as "finished".
+            if(has_data_to_send()) {
+               send_pending_encrypted_data(ec);
+            }
 
-         while(!native_handle()->is_active() && !ec) {
-            boost::asio::const_buffer read_buffer{input_buffer().data(), m_nextLayer.read_some(input_buffer(), ec)};
-            if(ec) {
+            // Once the underlying TLS implementation reports a complete and
+            // successful handshake we're done.
+            if(native_handle()->is_handshake_complete()) {
                return;
             }
 
-            process_encrypted_data(read_buffer, ec);
+            // Handle and report any TLS protocol errors that might have
+            // surfaced in a previous iteration. By postponing their handling we
+            // allow the stream to send a respective TLS alert to the peer before
+            // aborting the handshake.
+            handle_tls_protocol_errors(ec);
 
-            send_pending_encrypted_data(ec);
+            // If we don't have any encrypted data to send we attempt to read
+            // more data from the peer. This reports network errors immediately.
+            // TLS protocol errors result in an internal state change which is
+            // handled by `handle_tls_protocol_errors()` in the next iteration.
+            read_and_process_encrypted_data_from_peer(ec);
          }
+
+         BOTAN_ASSERT_NOMSG(ec.failed());
       }
 
       /**
@@ -251,14 +382,13 @@ class Stream {
        * @param completion_token The completion handler to be called when the handshake operation completes.
        *                         The completion signature of the handler must be: void(boost::system::error_code).
        */
-      template <typename CompletionToken>
-      auto async_handshake(Botan::TLS::Connection_Side side, CompletionToken&& completion_token) {
+      template <detail::basic_completion_token CompletionToken = default_completion_token>
+      auto async_handshake(Botan::TLS::Connection_Side side,
+                           CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code)>(
             [this](auto&& completion_handler, TLS::Connection_Side connection_side) {
                using completion_handler_t = std::decay_t<decltype(completion_handler)>;
 
-               BOOST_ASIO_HANDSHAKE_HANDLER_CHECK(completion_handler_t, completion_handler) type_check;
-
                boost::system::error_code ec;
                setup_native_handle(connection_side, ec);
 
@@ -270,11 +400,11 @@ class Stream {
       }
 
       //! @throws Not_Implemented
-      template <typename ConstBufferSequence, typename BufferedHandshakeHandler>
-      BOOST_ASIO_INITFN_RESULT_TYPE(BufferedHandshakeHandler, void(boost::system::error_code, std::size_t))
-      async_handshake(Connection_Side side, const ConstBufferSequence& buffers, BufferedHandshakeHandler&& handler) {
+      template <typename ConstBufferSequence, detail::basic_completion_token BufferedHandshakeHandler>
+      auto async_handshake(Connection_Side side,
+                           const ConstBufferSequence& buffers,
+                           BufferedHandshakeHandler&& handler) {
          BOTAN_UNUSED(side, buffers, handler);
-         BOOST_ASIO_HANDSHAKE_HANDLER_CHECK(BufferedHandshakeHandler, handler) type_check;
          throw Not_Implemented("buffered async handshake is not implemented");
       }
 
@@ -351,14 +481,12 @@ class Stream {
        * @param completion_token The completion handler to be called when the shutdown operation completes.
        *                         The completion signature of the handler must be: void(boost::system::error_code).
        */
-      template <typename CompletionToken>
-      auto async_shutdown(CompletionToken&& completion_token) {
+      template <detail::basic_completion_token CompletionToken = default_completion_token>
+      auto async_shutdown(CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code)>(
             [this](auto&& completion_handler) {
                using completion_handler_t = std::decay_t<decltype(completion_handler)>;
 
-               BOOST_ASIO_SHUTDOWN_HANDLER_CHECK(completion_handler_t, completion_handler) type_check;
-
                boost::system::error_code ec;
                try_with_error_code([&] { native_handle()->close(); }, ec);
 
@@ -390,31 +518,33 @@ class Stream {
        */
       template <typename MutableBufferSequence>
       std::size_t read_some(const MutableBufferSequence& buffers, boost::system::error_code& ec) {
-         if(has_received_data()) {
-            return copy_received_data(buffers);
+         // We read from the socket until either some error occured or we have
+         // decrypted at least one byte of application data.
+         while(!ec) {
+            // Some previous invocation of process_encrypted_data() generated
+            // application data in the output buffer that can now be returned.
+            if(has_received_data()) {
+               return copy_received_data(buffers);
+            }
+
+            // Handle and report any TLS protocol errors (including a
+            // close_notify) that might have surfaced in a previous iteration
+            // (in `read_and_process_encrypted_data_from_peer()`). This makes
+            // sure that all received application data was handed out to the
+            // caller before reporting an error (e.g. EOF at the end of the
+            // stream).
+            handle_tls_protocol_errors(ec);
+
+            // If we don't have any plaintext application data, yet, we attempt
+            // to read more data from the peer. This reports network errors
+            // immediately. TLS protocol errors result in an internal state
+            // change which is handled by `handle_tls_protocol_errors()` in the
+            // next iteration.
+            read_and_process_encrypted_data_from_peer(ec);
          }
 
-         boost::asio::const_buffer read_buffer{input_buffer().data(), m_nextLayer.read_some(input_buffer(), ec)};
-         if(ec) {
-            return 0;
-         }
-
-         process_encrypted_data(read_buffer, ec);
-
-         if(ec)  // something went wrong in process_encrypted_data()
-         {
-            return 0;
-         }
-
-         if(shutdown_received()) {
-            // we just received a 'close_notify' from the peer and don't expect any more data
-            ec = boost::asio::error::eof;
-         } else if(ec == boost::asio::error::eof) {
-            // we did not expect this disconnection from the peer
-            ec = StreamError::StreamTruncated;
-         }
-
-         return !ec ? copy_received_data(buffers) : 0;
+         BOTAN_ASSERT_NOMSG(ec.failed());
+         return 0;
       }
 
       /**
@@ -478,21 +608,21 @@ class Stream {
        *                         handler will be made as required. The completion signature of the handler must be:
        *                         void(boost::system::error_code, std::size_t).
        */
-      template <typename ConstBufferSequence, typename CompletionToken>
-      auto async_write_some(const ConstBufferSequence& buffers, CompletionToken&& completion_token) {
+      template <typename ConstBufferSequence,
+                detail::byte_size_completion_token CompletionToken = default_completion_token>
+      auto async_write_some(const ConstBufferSequence& buffers,
+                            CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code, std::size_t)>(
             [this](auto&& completion_handler, const auto& bufs) {
                using completion_handler_t = std::decay_t<decltype(completion_handler)>;
 
-               BOOST_ASIO_WRITE_HANDLER_CHECK(completion_handler_t, completion_handler) type_check;
-
                boost::system::error_code ec;
                tls_encrypt(bufs, ec);
 
                if(ec) {
                   // we cannot be sure how many bytes were committed here so clear the send_buffer and let the
                   // AsyncWriteOperation call the handler with the error_code set
-                  consume_send_buffer(m_core->send_buffer.size());
+                  m_core->send_buffer().consume(m_core->send_buffer().size());
                }
 
                detail::AsyncWriteOperation<completion_handler_t, Stream> op{
@@ -514,14 +644,14 @@ class Stream {
        * @param completion_token The completion handler to be called when the read operation completes. The completion
        *                         signature of the handler must be: void(boost::system::error_code, std::size_t).
        */
-      template <typename MutableBufferSequence, typename CompletionToken>
-      auto async_read_some(const MutableBufferSequence& buffers, CompletionToken&& completion_token) {
+      template <typename MutableBufferSequence,
+                detail::byte_size_completion_token CompletionToken = default_completion_token>
+      auto async_read_some(const MutableBufferSequence& buffers,
+                           CompletionToken&& completion_token = default_completion_token{}) {
          return boost::asio::async_initiate<CompletionToken, void(boost::system::error_code, std::size_t)>(
             [this](auto&& completion_handler, const auto& bufs) {
                using completion_handler_t = std::decay_t<decltype(completion_handler)>;
 
-               BOOST_ASIO_READ_HANDLER_CHECK(completion_handler_t, completion_handler) type_check;
-
                detail::AsyncReadOperation<completion_handler_t, Stream, MutableBufferSequence> op{
                   std::forward<completion_handler_t>(completion_handler), *this, bufs};
             },
@@ -535,7 +665,7 @@ class Stream {
       //!
       //! Note that we cannot m_core.is_closed_for_reading() because this wants to
       //! explicitly check that the peer sent close_notify.
-      bool shutdown_received() const { return m_core->shutdown_received; }
+      bool shutdown_received() const { return m_core->shutdown_received(); }
 
    protected:
       template <class H, class S, class M, class A>
@@ -545,93 +675,30 @@ class Stream {
       template <class H, class S, class A>
       friend class detail::AsyncHandshakeOperation;
 
-      /**
-       * @brief Helper class that implements TLS::Callbacks
-       *
-       * This class is provided to the stream's native_handle (TLS::Channel) and implements the callback
-       * functions triggered by the native_handle.
-       */
-      class StreamCore : public TLS::Callbacks {
-         public:
-            StreamCore(std::weak_ptr<Botan::TLS::Context> context) : shutdown_received(false), m_context(context) {}
-
-            virtual ~StreamCore() = default;
-
-            void tls_emit_data(std::span<const uint8_t> data) override {
-               send_buffer.commit(boost::asio::buffer_copy(send_buffer.prepare(data.size()),
-                                                           boost::asio::buffer(data.data(), data.size())));
-            }
-
-            void tls_record_received(uint64_t, std::span<const uint8_t> data) override {
-               receive_buffer.commit(boost::asio::buffer_copy(receive_buffer.prepare(data.size()),
-                                                              boost::asio::const_buffer(data.data(), data.size())));
-            }
-
-            bool tls_peer_closed_connection() override {
-               // Instruct the TLS implementation to reply with our close_notify to obtain
-               // the same behaviour for TLS 1.2 and TLS 1.3.
-               return true;
-            }
-
-            void tls_alert(TLS::Alert alert) override {
-               if(alert.type() == TLS::AlertType::CloseNotify) {
-                  shutdown_received = true;
-                  // Channel::process_alert will automatically write the corresponding close_notify response to the
-                  // send_buffer and close the native_handle after this function returns.
-               }
-            }
-
-            std::chrono::milliseconds tls_verify_cert_chain_ocsp_timeout() const override {
-               return std::chrono::milliseconds(1000);
-            }
-
-            void tls_verify_cert_chain(const std::vector<X509_Certificate>& cert_chain,
-                                       const std::vector<std::optional<OCSP::Response>>& ocsp_responses,
-                                       const std::vector<Certificate_Store*>& trusted_roots,
-                                       Usage_Type usage,
-                                       std::string_view hostname,
-                                       const TLS::Policy& policy) override {
-               auto ctx = m_context.lock();
-
-               if(ctx && ctx->has_verify_callback()) {
-                  ctx->get_verify_callback()(cert_chain, ocsp_responses, trusted_roots, usage, hostname, policy);
-               } else {
-                  Callbacks::tls_verify_cert_chain(cert_chain, ocsp_responses, trusted_roots, usage, hostname, policy);
-               }
-            }
-
-            bool shutdown_received;
-            boost::beast::flat_buffer receive_buffer;
-            boost::beast::flat_buffer send_buffer;
-
-         private:
-            std::weak_ptr<TLS::Context> m_context;
-      };
-
       const boost::asio::mutable_buffer& input_buffer() { return m_input_buffer; }
 
-      boost::asio::const_buffer send_buffer() const { return m_core->send_buffer.data(); }
+      boost::asio::const_buffer send_buffer() const { return m_core->send_buffer().data(); }
 
       //! @brief Check if decrypted data is available in the receive buffer
-      bool has_received_data() const { return m_core->receive_buffer.size() > 0; }
+      bool has_received_data() const { return m_core->receive_buffer().size() > 0; }
 
       //! @brief Copy decrypted data into the user-provided buffer
       template <typename MutableBufferSequence>
       std::size_t copy_received_data(MutableBufferSequence buffers) {
-         // Note: It would be nice to avoid this buffer copy. This could be achieved by equipping the StreamCore with
+         // Note: It would be nice to avoid this buffer copy. This could be achieved by equipping the CallbacksT with
          // the user's desired target buffer once a read is started, and reading directly into that buffer in tls_record
          // received. However, we need to deal with the case that the receive buffer provided by the caller is smaller
          // than the decrypted record, so this optimization might not be worth the additional complexity.
-         const auto copiedBytes = boost::asio::buffer_copy(buffers, m_core->receive_buffer.data());
-         m_core->receive_buffer.consume(copiedBytes);
+         const auto copiedBytes = boost::asio::buffer_copy(buffers, m_core->receive_buffer().data());
+         m_core->receive_buffer().consume(copiedBytes);
          return copiedBytes;
       }
 
       //! @brief Check if encrypted data is available in the send buffer
-      bool has_data_to_send() const { return m_core->send_buffer.size() > 0; }
+      bool has_data_to_send() const { return m_core->send_buffer().size() > 0; }
 
       //! @brief Mark bytes in the send buffer as consumed, removing them from the buffer
-      void consume_send_buffer(std::size_t bytesConsumed) { m_core->send_buffer.consume(bytesConsumed); }
+      void consume_send_buffer(std::size_t bytesConsumed) { m_core->send_buffer().consume(bytesConsumed); }
 
       /**
        * @brief Create the native handle.
@@ -643,11 +710,11 @@ class Stream {
        * @param ec Set to indicate what error occurred, if any.
        */
       void setup_native_handle(Connection_Side side, boost::system::error_code& ec) {
-         BOTAN_UNUSED(side);  // workaround: GCC 9 produces a warning claiming side is unused
-
          // Do not attempt to instantiate the native_handle when a custom (mocked) channel type template parameter has
          // been specified. This allows mocking the native_handle in test code.
          if constexpr(std::is_same<ChannelT, Channel>::value) {
+            BOTAN_STATE_CHECK(m_native_handle == nullptr);
+
             try_with_error_code(
                [&] {
                   if(side == Connection_Side::Client) {
@@ -672,6 +739,84 @@ class Stream {
          }
       }
 
+      /**
+       * The `Stream` has to distinguish from network-related issues (that are
+       * reported immediately) from TLS protocol errors, that must be retained
+       * and emitted once all legal application traffic received before is
+       * pushed to the downstream application.
+       *
+       * See also `process_encrypted_data()` and `StreamCallbacks::tls_alert()`
+       * where those TLS protocol errors are detected and retained for eventual
+       * handling in this method.
+       *
+       * See also https://github.com/randombit/botan/pull/3801 for a detailed
+       * description of the ASIO stream's state management.
+       *
+       * @param ec  this error code is set if we previously detected a TLS
+       *            protocol error.
+       */
+      void handle_tls_protocol_errors(boost::system::error_code& ec) {
+         if(ec) {
+            return;
+         }
+
+         // If we had raised an error while processing TLS records received from
+         // the peer, we expose that error here.
+         //
+         // See also `process_encrypted_data()`.
+         else if(auto error = error_from_us()) {
+            ec = error;
+         }
+
+         // If we had received a TLS alert from the peer, we expose that error
+         // here. See also `StreamCallbacks::tls_alert()` where such alerts
+         // would be detected and retained initially.
+         //
+         // Note that a close_notify is always a legal way for the peer to end a
+         // TLS session. When received during the handshake it typically means
+         // that the peer wanted to cancel the handshake for some reason not
+         // related to the TLS protocol.
+         else if(auto alert = alert_from_peer()) {
+            if(alert->type() == AlertType::CloseNotify) {
+               ec = boost::asio::error::eof;
+            } else {
+               ec = alert->type();
+            }
+         }
+      }
+
+      /**
+       * Reads TLS record data from the peer and forwards it to the native
+       * handle for processing. Note that @p ec will reflect network errors
+       * only. Any detected or received TLS protocol errors will be retained and
+       * must be handled by the downstream operation in due time by invoking
+       * `handle_tls_protocol_errors()`.
+       *
+       * @param ec  this error code might be populated with network-related errors
+       */
+      void read_and_process_encrypted_data_from_peer(boost::system::error_code& ec) {
+         if(ec) {
+            return;
+         }
+
+         // If we have received application data in a previous invocation, this
+         // data needs to be passed to the application first. Otherwise, it
+         // might get overwritten.
+         BOTAN_ASSERT(!has_received_data(), "receive buffer is empty");
+         BOTAN_ASSERT(!error_from_us() && !alert_from_peer(), "TLS session is healthy");
+
+         // If there's no existing error condition, read and process data from
+         // the peer and report any sort of network error. TLS related errors do
+         // not immediately cause an abort, they are checked in the invocation
+         // via `error_from_us()`.
+         boost::asio::const_buffer read_buffer{input_buffer().data(), m_nextLayer.read_some(input_buffer(), ec)};
+         if(!ec) {
+            process_encrypted_data(read_buffer);
+         } else if(ec == boost::asio::error::eof) {
+            ec = StreamError::StreamTruncated;
+         }
+      }
+
       /** @brief Synchronously write encrypted data from the send buffer to the next layer.
        *
        * If this function is called with an error code other than 'Success', it will do nothing and return 0.
@@ -703,35 +848,56 @@ class Stream {
        */
       template <typename ConstBufferSequence>
       void tls_encrypt(const ConstBufferSequence& buffers, boost::system::error_code& ec) {
+         // TODO: Once the TLS::Channel can deal with buffer sequences we can
+         //       save this copy. Until then we optimize for the smallest amount
+         //       of TLS records and flatten the boost buffer sequence.
+         std::vector<uint8_t> copy_buffer;
+         auto unpack = [&copy_buffer](const auto& bufs) -> std::span<const uint8_t> {
+            const auto buffers_in_sequence =
+               std::distance(boost::asio::buffer_sequence_begin(bufs), boost::asio::buffer_sequence_end(bufs));
+
+            if(buffers_in_sequence == 0) {
+               return {};
+            } else if(buffers_in_sequence == 1) {
+               const boost::asio::const_buffer buf = *boost::asio::buffer_sequence_begin(bufs);
+               return {static_cast<const uint8_t*>(buf.data()), buf.size()};
+            } else {
+               copy_buffer.resize(boost::asio::buffer_size(bufs));
+               boost::asio::buffer_copy(boost::asio::buffer(copy_buffer), bufs);
+               return copy_buffer;
+            }
+         };
+
          // NOTE: This is not asynchronous: it encrypts the data synchronously.
          // The data encrypted by native_handle()->send() is synchronously stored in the send_buffer of m_core,
          // but is not actually written to the wire, yet.
-         for(auto it = boost::asio::buffer_sequence_begin(buffers);
-             !ec && it != boost::asio::buffer_sequence_end(buffers);
-             it++) {
-            const boost::asio::const_buffer buffer = *it;
-            try_with_error_code(
-               [&] {
-                  native_handle()->send({static_cast<const uint8_t*>(buffer.data()), buffer.size()});
-               },
-               ec);
-         }
+         try_with_error_code([&] { native_handle()->send(unpack(buffers)); }, ec);
       }
 
       /**
-       * @brief Pass encrypted data to the native handle for processing.
+       * Pass encrypted data received from the peer to the native handle for
+       * processing. If the @p read_buffer contains coalesced TLS records, this
+       * might result in multiple TLS protocol state changes.
        *
-       * If an exception occurs while processing the data, an error code will be set.
+       * To allow the ASIO stream wrapper to disentangle those state changes
+       * properly, any TLS protocol errors are retained and must be handled by
+       * calling `handle_tls_protocol_errors()` in due time.
        *
        * @param read_buffer Input buffer containing the encrypted data.
-       * @param ec Set to indicate what error occurred, if any.
        */
-      void process_encrypted_data(const boost::asio::const_buffer& read_buffer, boost::system::error_code& ec) {
+      void process_encrypted_data(const boost::asio::const_buffer& read_buffer) {
+         BOTAN_ASSERT(!alert_from_peer() && !error_from_us(),
+                      "no one sent an alert before (no data allowed after that)");
+
+         // If the local TLS implementation generates an alert, we are notified
+         // with an exception that is caught in try_with_error_code(). The error
+         // code is retained and not handled directly. Stream operations will
+         // have to invoke `handle_tls_protocol_errors()` to handle them later.
          try_with_error_code(
             [&] {
                native_handle()->received_data({static_cast<const uint8_t*>(read_buffer.data()), read_buffer.size()});
             },
-            ec);
+            m_ec_from_last_read);
       }
 
       //! @brief Catch exceptions and set an error_code
@@ -748,19 +914,36 @@ class Stream {
          }
       }
 
+   private:
+      /**
+       * Returns any alert previously received from the peer. This may include
+       * close_notify. Once the peer has sent any alert, no more data must be
+       * read from the stream.
+       */
+      std::optional<Alert> alert_from_peer() const { return m_core->alert_from_peer(); }
+
+      /**
+       * Returns any error code produced by the local TLS implementation. This
+       * will _not_ include close_notify. Once our TLS stack has reported an
+       * error, no more data must be written to the stream. The peer will receive
+       * the error as a TLS alert.
+       */
+      boost::system::error_code error_from_us() const { return m_ec_from_last_read; }
+
+   protected:
       std::shared_ptr<Context> m_context;
       StreamLayer m_nextLayer;
 
-      std::shared_ptr<StreamCore> m_core;
+      std::shared_ptr<StreamCallbacks> m_core;
       std::unique_ptr<ChannelT> m_native_handle;
+      boost::system::error_code m_ec_from_last_read;
 
       // Buffer space used to read input intended for the core
       std::vector<uint8_t> m_input_buffer_space;
       const boost::asio::mutable_buffer m_input_buffer;
 };
 
-}  // namespace TLS
-}  // namespace Botan
+}  // namespace Botan::TLS
 
-#endif  // BOOST_VERSION
+#endif
 #endif  // BOTAN_ASIO_STREAM_H_
